@@ -45,9 +45,9 @@ session_impl::session_impl(
       io_service_(io_service),
       resolver_(io_service),
       deadline_(io_service),
-      ping_(io_service),
       connect_timeout_(connect_timeout),
       read_timeout_(boost::posix_time::seconds(60)),
+      ping_(io_service),
       session_(nullptr),
       data_pending_(nullptr),
       data_pendinglen_(0),
@@ -84,7 +84,6 @@ void session_impl::start_resolve(const std::string &host,
                           });
 
   deadline_.async_wait(std::bind(&session_impl::handle_deadline, self));
-  start_ping();
 }
 
 void session_impl::handle_deadline() {
@@ -135,6 +134,8 @@ void session_impl::connected(tcp::resolver::iterator endpoint_it) {
   do_write();
   do_read();
 
+  start_ping();
+
   auto &connect_cb = on_connect();
   if (connect_cb) {
     connect_cb(endpoint_it);
@@ -472,13 +473,13 @@ stream *session_impl::create_push_stream(int32_t stream_id) {
 }
 
 std::unique_ptr<stream> session_impl::create_stream() {
-  return make_unique<stream>(this);
+  return std::make_unique<stream>(this);
 }
 
 const request *session_impl::submit(boost::system::error_code &ec,
                                     const std::string &method,
                                     const std::string &uri, generator_cb cb,
-                                    header_map h) {
+                                    header_map h, priority_spec prio) {
   ec.clear();
 
   if (stopped_) {
@@ -558,7 +559,7 @@ const request *session_impl::submit(boost::system::error_code &ec,
     prdptr = &prd;
   }
 
-  auto stream_id = nghttp2_submit_request(session_, nullptr, nva.data(),
+  auto stream_id = nghttp2_submit_request(session_, prio.get(), nva.data(),
                                           nva.size(), prdptr, strm.get());
   if (stream_id < 0) {
     ec = make_error_code(static_cast<nghttp2_error>(stream_id));
@@ -755,4 +756,4 @@ void session_impl::read_timeout(const boost::posix_time::time_duration &t) {
 
 } // namespace client
 } // namespace asio_http2
-} // nghttp2
+} // namespace nghttp2
