@@ -64,7 +64,8 @@ public:
   // migrate to another Http2Session object.
   virtual bool poolable() const { return false; }
 
-  virtual DownstreamAddrGroup *get_downstream_addr_group() const;
+  virtual const std::shared_ptr<DownstreamAddrGroup> &
+  get_downstream_addr_group() const;
   virtual DownstreamAddr *get_addr() const;
 
   int send();
